require File.expand_path('../pluggable', __FILE__)
require File.expand_path('../parser', __FILE__)
require File.expand_path('../options', __FILE__)
require File.expand_path('../app_defaults', __FILE__)
require File.expand_path('../commands', __FILE__)

module Scripting
  class Application
    extend Pluggable

    def initialize
      clear_switches!
      clear_options!
      clear_work!
      clear_help!
    end

    def context
      self
    end

    def describe &blk
      instance_eval &blk if block_given?
      self
    end

    def clear_switches!
      @switches = Parser.new(self)
    end

    def switches &blk
      @switches.instance_eval &blk if block_given?
      @switches
    end

    def clear_options!
      @options = Options.new
    end

    def options &blk
      @options.describe &blk if block_given?
      @options
    end

    def clear_work!
      @work = []
    end

    def work &blk
      @work << blk if block_given?
      @work
    end

    def clear_help!
      @help = [lambda { $stderr.puts @switches }]
    end

    def help &blk
      @help << blk if block_given?
      @help
    end

    def help!
      @help.each(&:call)
    end

    def parse! args
      switches.parse! args
      args
    end

    def run! *args
      if args.first.kind_of? Array
        args.flatten!
      end
      parse! args
      work.each { |work| instance_exec(*args, &work) }
    end
  end

  # Creates a new Scripting::Application class instance receiving a block
  # used to describe options, command line switches, and other traits.
  #
  #   my_app = Scripting::create_application do
  #     plugin Scripting::AppDefaults
  #
  #     options do
  #       message = "Hello World"
  #       stderr = false
  #       stdout = true
  #     end
  #
  #     switches do
  #       on('--message=TEXT (default: #{options.message})') { |opt| options.message = opt }
  #       on('--[no]-stderr') { |opt| options.stderr = opt }
  #       on('--[no]-stdout') { |opt| options.stdout = opt }
  #     end
  #
  #     work do
  #       $stderr.puts options.message if options.stderr?
  #       $stdout.puts options.message if options.stdout?
  #     end
  #   end
  #
  def self.create_application &blk
    app = Class.new(Application).new
    app.describe &blk
    app
  end
end
